/*
 * Java
 * 
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilePermission;
import java.io.FilenameFilter;
import java.io.IOException;
import java.security.Permission;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/**
 *
 */
public class TestFilePermission {

	public static final Class C = TestFilePermission.class;

	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();
		CheckHelper.startCheck(C);
		checkCanonicalActions();
		checkActionsKO();
		checkEquals();
		checkImplies();
		checkPermissionChecksRead();
		checkPermissionChecksWrite();
		checkPermissionChecksDelete();
		checkPermissionChecksExec();
		CheckHelper.endCheck(C);
	}


	private static void checkPermissionChecksWrite() {
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.canWrite();
			}
		}, "write");

		checkPermissionCheck(new RunnableIO(getNonExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.createNewFile();
			}
		}, "write");

		checkPermissionCheck(new RunnableIO(getNonExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.mkdir();
			}
		}, "write");

		checkPermissionCheck(new RunnableIO(getNonExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.mkdirs();
			}
		}, "write");

		final File f2 = getNonExistingFile();
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.renameTo(f2);
			}
		}, "write");

		checkPermissionCheck(new RunnableIO(null) {

			@Override
			protected void runIO(File file) throws IOException{
				File.createTempFile("test", ".test");
			}
		}, "write");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.setLastModified(0);
			}
		}, "write");
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.setReadOnly();
			}
		}, "write");
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.setWritable(true);
			}
		}, "write");
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.setWritable(false, false);
			}
		}, "write");
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.setReadable(false);
			}
		}, "write");
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.setReadable(false, false);
			}
		}, "write");
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.setExecutable(false);
			}
		}, "write");
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.setExecutable(false, false);
			}
		}, "write");



		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@SuppressWarnings("resource")
			@Override
			protected void runIO(File file) throws IOException{
				new FileOutputStream(file);
			}
		}, "write");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@SuppressWarnings("resource")
			@Override
			protected void runIO(File file) throws IOException{
				new FileOutputStream(file.getAbsolutePath());
			}
		}, "write");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@SuppressWarnings("resource")
			@Override
			protected void runIO(File file) throws IOException{
				new FileOutputStream(file.getAbsolutePath(), true);
			}
		}, "write");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@SuppressWarnings("resource")
			@Override
			protected void runIO(File file) throws IOException{
				new FileOutputStream(file, true);
			}
		}, "write");
	}

	private static void checkPermissionChecksDelete() {
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.delete();
			}
		}, "delete");
	}

	private static void checkPermissionChecksExec() {
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.canExecute();
			}
		}, "execute");
	}

	/**
	 *
	 */
	private static void checkPermissionChecksRead() {
		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			protected void runIO(File file) throws IOException{
				file.getCanonicalPath();
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.getCanonicalFile();
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.canRead();
			}
		}, "read");


		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.exists();
			}
		}, "read");


		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.isDirectory();
			}
		}, "read");


		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.isFile();
			}
		}, "read");


		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.isHidden();
			}
		}, "read");


		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.lastModified();
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.list();
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.list(new FilenameFilter() {

					@Override
					public boolean accept(File dir, String name) {
						return true;
					}
				});
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.listFiles();
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.listFiles(new FileFilter() {

					@Override
					public boolean accept(File pathname) {
						return true;
					}
				});
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.listFiles(new FilenameFilter() {

					@Override
					public boolean accept(File dir, String name) {
						return true;
					}
				});
			}
		}, "read");


		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.getFreeSpace();
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.getTotalSpace();
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@Override
			public void runIO(File file) throws IOException{
				file.getUsableSpace();
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@SuppressWarnings("resource")
			@Override
			protected void runIO(File file) throws IOException{
				new FileInputStream(file);
			}
		}, "read");

		checkPermissionCheck(new RunnableIO(getExistingFile()) {

			@SuppressWarnings("resource")
			@Override
			protected void runIO(File file) throws IOException{
				new FileInputStream(file.getAbsolutePath());
			}
		}, "read");

	}

	/**
	 * @return
	 */
	private static File getExistingFile() {
		try { return File.createTempFile("test", "test"); }
		catch (IOException e1) {
			CheckHelper.check(C, "checkPermissionChecks-tmpfile", false);
			return null;
		}
	}

	private static File getNonExistingFile() {
		File f;
		try { f = File.createTempFile("test", "test"); }
		catch (IOException e1) {
			CheckHelper.check(C, "checkPermissionChecks-tmpfile", false);
			return null;
		}
		boolean result = f.delete();
		CheckHelper.check(C, "checkPermissionChecks-getNonExistingFileDelete", result);
		return f;
	}

	private static void checkPermissionCheck(final RunnableIO runnable, final String actions) {
		System.setSecurityManager(new SecurityManager(){
			@Override
			public void checkPermission(Permission perm) {
				if(perm instanceof FilePermission){
					if(perm.getActions().equals("read") && !actions.equals("read")){
						// Skip internal J2SE read permission when building the permission (getCanonicalFile())
						return;
					}
					if(!perm.getName().endsWith("test")){
						// Skip internal J2SE tmp file (SecureRandom...)
						return;
					}
					FilePermission fp = (FilePermission)perm;
					CheckHelper.check(C, "checkPermissionCheck-FilePermissionActions", actions, fp.getActions());
					throw new SecurityException();
				}
			}
		});

		try{
			runnable.run();
		}
		catch(SecurityException e){
			CheckHelper.check(C, "checkPermissionCheck-SE", true);
			return;
		}
		finally{
			System.setSecurityManager(null);
		}
		CheckHelper.check(C, "checkPermissionCheck-CCN", false);
	}

	private static void checkEquals() {
		checkEqualsOK("/aaa", "read,execute", "/aaa", "read,execute");
		checkEqualsOK("/aaa", "read,execute", "/aaa", "execute,read");

		checkEqualsKO("/aaa", "read", "/aaa", "execute");
		checkEqualsKO("/aaa", "read", "/bbb", "read");
	}

	private static void checkEqualsOK(String path1, String actions1, String path2, String actions2) {
		FilePermission p1 = new FilePermission(path1, actions1);
		FilePermission p2 = new FilePermission(path2, actions2);
		CheckHelper.check(C, "checkEqualsOK", p1.equals(p2));
	}

	private static void checkEqualsKO(String path1, String actions1, String path2, String actions2) {
		FilePermission p1 = new FilePermission(path1, actions1);
		FilePermission p2 = new FilePermission(path2, actions2);
		CheckHelper.check(C, "checkEqualsKO", !p1.equals(p2));
	}

	private static void checkImpliesOK(String path1, String actions1, String path2, String actions2) {
		FilePermission p1 = new FilePermission(path1, actions1);
		FilePermission p2 = new FilePermission(path2, actions2);
		CheckHelper.check(C, "checkImpliesOK", p1.implies(p2));
	}

	private static void checkImpliesKO(String path1, String actions1, String path2, String actions2) {
		FilePermission p1 = new FilePermission(path1, actions1);
		FilePermission p2 = new FilePermission(path2, actions2);
		CheckHelper.check(C, "checkImpliesKO", !p1.implies(p2));
	}

	private static void checkImplies() {
		// tests from examples explained in implies javadoc
		checkImpliesOK("/tmp", "read,write", "/tmp", "read");
		checkImpliesKO("/tmp", "read", "/tmp", "read,write");

		checkImpliesOK("/tmp/*", "read", "/tmp/foo", "read");
		checkImpliesKO("/tmp/foo", "read", "/tmp/*", "read");
	}

	private static void checkActionsKO() {
		checkActionsKO(null);
		checkActionsKO("");
		checkActionsKO("read,");
		checkActionsKO("wrote");
		checkActionsKO(",");
		checkActionsKO(",,read");
		checkActionsKO(",,read");
	}

	private static void checkActionsKO(String actions) {
		try{
			new FilePermission("/aaa", actions);
		}
		catch(IllegalArgumentException e){
			CheckHelper.check(C, "checkActionsKO-IAE-"+actions, true);
			return;
		}
		CheckHelper.check(C, "checkActionsKO-CCN-"+actions, false);
	}

	private static void checkCanonicalActions() {
		checkActionsOK("read", "read");
		checkActionsOK("write", "write");
		checkActionsOK("execute", "execute");
		checkActionsOK("delete", "delete");
		checkActionsOK("readlink", "readlink");

		checkActionsOK("READ", "read");
		checkActionsOK("WRITE", "write");
		checkActionsOK("EXECUTE", "execute");
		checkActionsOK("DELETE", "delete");
		checkActionsOK("READLINK", "readlink");

		checkActionsOK("read,write", "read,write");
		checkActionsOK("write,read", "read,write");
		checkActionsOK("write,execute", "write,execute");
		checkActionsOK("execute,write", "write,execute");
		checkActionsOK("execute,delete", "execute,delete");
		checkActionsOK("delete,execute", "execute,delete");
		checkActionsOK("delete,readlink", "delete,readlink");
		checkActionsOK("readlink,delete", "delete,readlink");



		checkActionsOK("readlink,DELETE", "delete,readlink");


		checkActionsOK("read,write,read", "read,write");
		checkActionsOK("write,read,write", "read,write");
	}

	private static void checkActionsOK(String actions, String expectedCanonicalActions) {
		FilePermission p = new FilePermission("/aaa", actions);
		CheckHelper.check(C, "checkActionsOK-"+actions, p.getActions(), expectedCanonicalActions);
	}

}

abstract class RunnableIO implements Runnable{

	private final File file;
	public RunnableIO(File file) {
		this.file = file;
	}
	@Override
	public void run() {
		try {
			runIO(file);
		} catch (IOException e) {

		}
	}
	/**
	 * @param file2
	 */
	protected abstract void runIO(File file) throws IOException;
	/**
	 * @return
	 */
	public File getFile() {
		return file;
	}

}
