/*
 * Java
 * 
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

import ej.lang.Resource;
import ej.lang.ResourceManager;

/**
 *
 */
public class TestResources01 {

	public static Class C = TestResources01.class;

	public static void main(String[] args) throws IOException{
		FileSystemCleaner.cleanFS();
		checkNoResourceLeak();
		checkFileInputStreamOPEN_CLOSE();
		checkFileInputStreamOPEN_RECLAIM();
		checkFileOutputStreamOPEN_CLOSE();
		checkFileOutputStreamOPEN_RECLAIM();
		CheckHelper.endCheck(C);
	}

	private static void checkFileInputStreamOPEN_RECLAIM() {
		MyResourceManager rm = new MyResourceManager();
		ResourceManager.setResourceManager(rm);

		File f = getExistingFile();
		final Object lock = new Object();
		final boolean[] result = new boolean[1];
		try{
			final FileInputStream fis = new FileInputStream(f);

			CheckHelper.check(TestResources01.C, "checkFileInputStreamOPEN_RECLAIM-ResourceOpen", rm.resourcesVect.size(), 1);
			new Thread(new Runnable() {

				@Override
				public void run() {
					try{
						while(true){
							fis.available();
						}
					}
					catch(IOException e){
						synchronized (lock) {
							result[0] = true;
							lock.notify();
						}
					}
				}
			}).start();

			try { Thread.sleep(1000); }
			catch (InterruptedException e) { throw new AssertionError(e); }

			CheckHelper.check(TestResources01.C, "checkFileInputStreamOPEN_RECLAIM-threadRunning", !result[0]);
			rm.resourcesVect.get(0).reclaim();

			synchronized (lock) {
				while(!result[0]){
					try { lock.wait(); }
					catch (InterruptedException e1) {}
				}
			}
		}
		catch(IOException e){
			CheckHelper.check(C, "checkFileInputStreamOPEN_RECLAIM-IO", false);
		}
		CheckHelper.check(TestResources01.C, "checkFileInputStreamOPEN_RECLAIM-ResourceClosed", rm.resourcesVect.size(), 0);

	}

	private static void checkFileOutputStreamOPEN_RECLAIM() {
		MyResourceManager rm = new MyResourceManager();
		ResourceManager.setResourceManager(rm);

		File f = getExistingFile();
		final Object lock = new Object();
		final boolean[] result = new boolean[1];
		try{
			final FileOutputStream fos = new FileOutputStream(f);

			CheckHelper.check(TestResources01.C, "checkFileOutputStreamOPEN_RECLAIM-ResourceOpen", rm.resourcesVect.size(), 1);
			new Thread(new Runnable() {

				@Override
				public void run() {
					try{
						while(true){
							fos.write(0);
							try { Thread.sleep(100); }
							catch (InterruptedException e) { throw new AssertionError(e); }
						}
					}
					catch(IOException e){
						synchronized (lock) {
							result[0] = true;
							lock.notify();
						}
					}
				}
			}).start();

			try { Thread.sleep(1000); }
			catch (InterruptedException e) { throw new AssertionError(e); }

			CheckHelper.check(TestResources01.C, "checkFileOutputStreamOPEN_RECLAIM-threadRunning", !result[0]);
			rm.resourcesVect.get(0).reclaim();

			synchronized (lock) {
				while(!result[0]){
					try { lock.wait(); }
					catch (InterruptedException e1) {}
				}
			}
		}
		catch(IOException e){
			CheckHelper.check(C, "checkFileOutputStreamOPEN_RECLAIM-IO", false);
		}
		CheckHelper.check(TestResources01.C, "checkFileOutputStreamOPEN_RECLAIM-ResourceClosed", rm.resourcesVect.size(), 0);

	}

	private static void checkFileInputStreamOPEN_CLOSE() {
		MyResourceManager rm = new MyResourceManager();
		ResourceManager.setResourceManager(rm);

		// test OPEN/CLOSE
		File f = getExistingFile();
		try(FileInputStream fis = new FileInputStream(f)){
			CheckHelper.check(TestResources01.C, "checkFileInputStreamOPEN_CLOSE-ResourceOpen", rm.resourcesVect.size(), 1);
		}
		catch(IOException e){
			CheckHelper.check(C, "checkFileInputStreamOPEN_CLOSE-IO", false);
		}
		CheckHelper.check(TestResources01.C, "checkFileInputStreamOPEN_CLOSE-ResourceClosed", rm.resourcesVect.size(), 0);

	}

	private static void checkFileOutputStreamOPEN_CLOSE() {
		MyResourceManager rm = new MyResourceManager();
		ResourceManager.setResourceManager(rm);

		// test OPEN/CLOSE
		File f = getExistingFile();
		try(FileOutputStream fos = new FileOutputStream(f)){
			CheckHelper.check(TestResources01.C, "checkFileOutputStreamOPEN_CLOSE-ResourceOpen", rm.resourcesVect.size(), 1);
		}
		catch(IOException e){
			CheckHelper.check(C, "checkFileOutputStreamOPEN_CLOSE-IO", false);
		}
		CheckHelper.check(TestResources01.C, "checkFileOutputStreamOPEN_CLOSE-ResourceClosed", rm.resourcesVect.size(), 0);

	}

	private static File getExistingFile() {
		try { return File.createTempFile("test", "test"); }
		catch (IOException e1) {
			CheckHelper.check(C, "checkPermissionChecks-tmpfile", false);
			return null;
		}
	}

	private static File getBasedir(){
		File[] files = File.listRoots();
		File basedir = files[0];
		return basedir;
	}

	private static void checkNoResourceLeak()  throws IOException{
		ResourceManager defaultRm = ResourceManager.getResourceManager();
		MyResourceManager rm = new MyResourceManager();

		ResourceManager.setResourceManager(rm);

		File basedir = getBasedir();
		basedir.listFiles();
		CheckHelper.check(TestResources01.C, "NoResourceLeak-list", rm.resourcesVect.size(), 0);


		File testfile = File.createTempFile("test", "test");
		FileOutputStream fos = new FileOutputStream(testfile);
		CheckHelper.check(TestResources01.C, "ResourceOpen", rm.resourcesVect.size(), 1);



		ResourceManager.setResourceManager(defaultRm);
	}

}
class MyResourceManager extends ResourceManager{

	ArrayList<Resource> resourcesVect = new ArrayList<>();

	@Override
	public void resourceCreated(Resource resource) {
		resourcesVect.add(resource);
	}

	@Override
	public void resourceReclaimed(Resource resource) {
		CheckHelper.check(TestResources01.C, "resourceRemoved", resourcesVect.remove(resource));
	}

}