/*
* Copyright 2015-2020 MicroEJ Corp. All rights reserved.
* This library is provided in source code for use, modification and test, subject to license terms.
* Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
*/
package com.microej.fs.tests.constructors;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/** Test {@link FileInputStream}. */
public class TestFileInputStream {

	/** Directory that exists. */
	private static File ExistingDir;
	/** Directory that does not exist. */
	private static File MissingDir;

	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();

		CheckHelper.startCheck(TestFileInputStream.class);
		initTests();

		try{
			testConstructorString();
			testConstructorFile();
			testAvailable();
			testReadEmptyFile();
			testReadByte();
			testReadArray();
			testReadArrayIntInt();
			testReadArrayIntIntInvalid();
		}
		catch(IOException e){

		}

		CheckHelper.endCheck(TestFileInputStream.class);
	}

	private static void initTests(){
		ExistingDir = new File("existing"+TestFileInputStream.class.getSimpleName());
		ExistingDir.mkdirs();
		CheckHelper.check(TestFileInputStream.class, "Existing dir must exist", ExistingDir.exists());
		//Delete all the file of the directory
		for(File files : ExistingDir.listFiles()){
			files.delete();
		}

		MissingDir = new File("missing"+TestFileInputStream.class.getSimpleName());
		CheckHelper.check(TestFileInputStream.class, "Missing dir must not exist", !MissingDir.exists());

	}

	/** Tests {@link FileInputStream#FileInputStream(String)} */
	public static void testConstructorString() throws IOException{

		// File that does not exist in an existing directory
		try {
			FileInputStream fis = new FileInputStream(ExistingDir.getAbsolutePath()+File.separator+"testString1");
			CheckHelper.check(TestFileInputStream.class, "No IOException(1)", false);
		}
		catch(IOException e){
		}

		// File that does not exist in a directory that does not exist
		try {
			FileInputStream fis = new FileInputStream(MissingDir.getAbsolutePath()+File.separator+"testString2");
			CheckHelper.check(TestFileInputStream.class, "No IOException(2)", false);
		}
		catch(IOException e){
		}

		// File that exists
		File file = File.createTempFile(TestFileInputStream.class.getSimpleName(), null);
		try {
			FileInputStream fis = new FileInputStream(file.getAbsolutePath());
			fis.close();
		}
		catch(IOException e){
			CheckHelper.check(TestFileInputStream.class, "IOException(2)", false);
		}
		file.delete();
	}

	/** Tests {@link FileInputStream#FileInputStream(File)} */
	public static void testConstructorFile() throws IOException{

		// File that does not exist in an existing directory
		try {
			FileInputStream fis = new FileInputStream(new File(ExistingDir, "testConstructorFile1"));
			CheckHelper.check(TestFileInputStream.class, "No IOException(1)", false);
		}
		catch(IOException e){
		}

		// File that does not exist in a directory that does not exist
		try {
			FileInputStream fis = new FileInputStream(new File(MissingDir, "testConstructorFile2"));
			CheckHelper.check(TestFileInputStream.class, "No IOException(2)", false);
		}
		catch(IOException e){
		}

		// File that exists
		File file = File.createTempFile(TestFileInputStream.class.getSimpleName(), null);
		try {
			FileInputStream fis = new FileInputStream(file);
			fis.close();
		}
		catch(IOException e){
			CheckHelper.check(TestFileInputStream.class, "IOException(2)", false);
		}
		file.delete();
	}

	/**
	 * Tests read on empty file
	 */
	public static void testReadEmptyFile() throws IOException {
		// Read empty file
		File file = File.createTempFile(TestFileInputStream.class.getSimpleName(), null);
		try(FileInputStream fis = new FileInputStream(file)){
			int res;

			res = fis.read();
			CheckHelper.check(TestFileInputStream.class, "Read() on empty file must return -1", res, -1);

			res = fis.read(new byte[10]);
			CheckHelper.check(TestFileInputStream.class, "Read(byte[]) on empty file must return -1", res, -1);

			res = fis.read(new byte[0]);
			CheckHelper.check(TestFileInputStream.class, "Read(byte[]) on empty file with empty array must return 0", res, 0);

			res = fis.read(new byte[10], 0, 5);
			CheckHelper.check(TestFileInputStream.class, "Read(byte[],int,int) on empty file must return -1", res, -1);

			res = fis.read(new byte[10], 0, 0);
			CheckHelper.check(TestFileInputStream.class, "Read(byte[],int,int) on empty file with 0 length must return 0", res, 0);
		}

		file.delete();
	}

	/** Tests {@link FileInputStream#read()} */
	public static void testReadByte() throws IOException {
		File file = File.createTempFile(TestFileInputStream.class.getSimpleName(), null);

		try(FileOutputStream fos = new FileOutputStream(file)){
			fos.write(new byte[]{-1, -100, 10, 20, 0, 127, 44});
		}

		// Read not empty file : read()
		try(FileInputStream fis = new FileInputStream(file)){
			int res;

			res = fis.read();
			CheckHelper.check(TestFileInputStream.class, "Read() 1", res, 255);
			res = fis.read();
			CheckHelper.check(TestFileInputStream.class, "Read() 2", res, 156);
			res = fis.read();
			CheckHelper.check(TestFileInputStream.class, "Read() 3", res, 10);
			res = fis.read();
			CheckHelper.check(TestFileInputStream.class, "Read() 4", res, 20);
			res = fis.read();
			CheckHelper.check(TestFileInputStream.class, "Read() 5", res, 0);
			res = fis.read();
			CheckHelper.check(TestFileInputStream.class, "Read() 6", res, 127);
			res = fis.read();
			CheckHelper.check(TestFileInputStream.class, "Read() 7", res, 44);
			for(int i=100 ; --i>=0 ; ){
				res = fis.read();
				CheckHelper.check(TestFileInputStream.class, "Read() EOF "+i, res, -1);
			}
		}

		file.delete();
	}

	/** Tests {@link FileInputStream#read(byte[])} */
	public static void testReadArray() throws IOException {
		File file = File.createTempFile(TestFileInputStream.class.getSimpleName(), null);

		byte[] data1 = new byte[]{-1, -100, 10, 20, 0, 127, 44};
		byte[] data2 = new byte[]{1, 2, 3, 4};
		byte[] data3 = new byte[]{10, 20, 0, 0};

		try(FileOutputStream fos = new FileOutputStream(file)){
			fos.write(data1);
			fos.write(data2);
			fos.write(data3, 0, 2);
		}

		// Read not empty file : read()
		try(FileInputStream fis = new FileInputStream(file)){

			checkRead(fis, data1, "Read(byte[]) 1");
			checkRead(fis, new byte[0], "Read(byte[]) 2");
			checkRead(fis, data2, "Read(byte[]) 3");
			checkRead(fis, new byte[0], "Read(byte[]) 4");
			checkRead(fis, data3, 2, "Read(byte[]) 5");

			byte[] data = new byte[1];
			for(int i=100 ; --i>=0 ; ){
				int res = fis.read(data);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[]) EOF "+i, res, -1);
			}

			checkRead(fis, new byte[0], "Read(byte[]) 6");
		}

		file.delete();
	}

	/** Tests {@link FileInputStream#read(byte[],int,int)} */
	public static void testReadArrayIntInt() throws IOException {
		File file = File.createTempFile(TestFileInputStream.class.getSimpleName(), null);

		byte[] data1 = new byte[]{-1, -100, 10, 20, 0, 127, 44};
		byte[] data2 = new byte[]{1, 2, 3, 4};
		byte[] data3 = new byte[]{20, 30};

		try(FileOutputStream fos = new FileOutputStream(file)){
			fos.write(data1);
			fos.write(data2);
			fos.write(data3);
		}

		// Read not empty file : read()
		try(FileInputStream fis = new FileInputStream(file)){

			int offset = 12;
			int length;
			byte[] dataTest;

			length = data1.length;
			dataTest = new byte[length+20];
			System.arraycopy(data1, 0, dataTest, offset, length);
			checkRead(fis, dataTest, offset, length, "Read(byte[],int,int) 1");

			dataTest = new byte[length+20];
			checkRead(fis, dataTest, offset, 0, "Read(byte[],int,int) 2");

			length = data2.length;
			dataTest = new byte[length+20];
			System.arraycopy(data2, 0, dataTest, offset, length);
			checkRead(fis, dataTest, offset, length, "Read(byte[],int,int) 3");

			dataTest = new byte[length+20];
			checkRead(fis, dataTest, offset, 0, "Read(byte[],int,int) 4");

			// Ask to read more than available
			byte[] expectedData3 = new byte[]{10, 20, 30, 40};
			byte[] readData3 = new byte[]{10, 0, 0, 40};
			int nbRead = fis.read(readData3, 1, 3);
			CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) nbRead (5)", nbRead, 2);
			for(int i=0 ; i<expectedData3.length ; i++){
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) (5) invalid content "+i,  readData3[i], expectedData3[i]);
			}

			byte[] data = new byte[10];
			for(int i=100 ; --i>=0 ; ){
				int res = fis.read(data, 5, 1);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) EOF "+i, res, -1);
			}

			dataTest = new byte[length+20];
			checkRead(fis, dataTest, offset, 0, "Read(byte[],int,int) 6");
		}

		file.delete();
	}

	/** Tests {@link FileInputStream#read(byte[],int,int)} with invalid length, offset, etc. */
	public static void testReadArrayIntIntInvalid() throws IOException {
		File file = File.createTempFile(TestFileInputStream.class.getSimpleName(), null);

		try(FileOutputStream fos = new FileOutputStream(file)){
			fos.write("toubidou".getBytes());
		}

		try(FileInputStream fis = new FileInputStream(file)){

			byte[] data = new byte[5];
			try{
				fis.read(data, 6, 1);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) invalid offset (1) ", false);
			}
			catch(IndexOutOfBoundsException e){
				// ok
			}

			try{
				fis.read(data, 1, -1);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) negative length (1) ", false);
			}
			catch(IndexOutOfBoundsException e){
				// ok
			}

			try{
				fis.read(data, 1, 6);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) invalid length (1) ", false);
			}
			catch(IndexOutOfBoundsException e){
				// ok
			}

			try{
				fis.read(data, 1, Integer.MAX_VALUE);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) overflow (1) ", false);
			}
			catch(IndexOutOfBoundsException e){
				// ok
			}

			try{
				fis.read(data, 100, 0);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) invalid offset but length is 0 (1) ", false);
			}
			catch(IndexOutOfBoundsException e){
				// ok
			}

		}
		file.delete();
	}

	/**
	 * Test that we can read the correct data from the given {@link FileInputStream}
	 */
	private static void checkRead(FileInputStream fis, byte[] data, String message) throws IOException{
		checkRead(fis, data, data.length, message);
	}

	/**
	 * Test that we can read the correct data from the given {@link FileInputStream}
	 */
	private static void checkRead(FileInputStream fis, byte[] data, int expectedLength, String message) throws IOException{
		int res;
		byte[] readData = new byte[data.length];
		res = fis.read(readData);
		CheckHelper.check(TestFileInputStream.class, message+": check nb read", res, expectedLength);
		for(int i=0 ; i<data.length ; i++){
			CheckHelper.check(TestFileInputStream.class, message+": invalid content "+i,  readData[i], data[i]);
		}
	}

	/**
	 * Test that we can read the correct data from the given {@link FileInputStream}
	 */
	private static void checkRead(FileInputStream fis, byte[] data, int offset, int length, String message) throws IOException{
		int res;
		byte[] readData = new byte[data.length];
		res = fis.read(readData, offset, length);
		CheckHelper.check(TestFileInputStream.class, message+": check nb read", res, length);
		for(int i=0 ; i<data.length ; i++){
			CheckHelper.check(TestFileInputStream.class, message+": invalid content "+i,  readData[i], data[i]);
		}
	}

	/**
	 * Tests {@link FileInputStream#available()}.
	 */
	public static void testAvailable() throws IOException {
		File file = File.createTempFile(TestFileInputStream.class.getSimpleName(), null);

		// Available on empty file
		try(FileInputStream fis = new FileInputStream(file)){
			int available = fis.available();
			CheckHelper.check(TestFileInputStream.class, "Available for empty file must be 0 (1)", available, 0);
			int res = fis.read();
			CheckHelper.check(TestFileInputStream.class, "Read empty file must return -1", res, -1);
			available = fis.available();
			CheckHelper.check(TestFileInputStream.class, "Available for empty file must be 0 (2)", available, 0);
		}

		try(FileOutputStream fos = new FileOutputStream(file)){
			fos.write("toubidou".getBytes());
		}

		// Available on non empty file
		try(FileInputStream fis = new FileInputStream(file)){
			int available = fis.available();
			CheckHelper.check(TestFileInputStream.class, "Available (1)", available, 8);
			int res = fis.read();
			CheckHelper.check(TestFileInputStream.class, "Read correct byte", res, 't');

			available = fis.available();
			CheckHelper.check(TestFileInputStream.class, "Available (2)", available, 7);
			available = fis.available();
			CheckHelper.check(TestFileInputStream.class, "Available (3)", available, 7);

			res = fis.read(new byte[7]);
			CheckHelper.check(TestFileInputStream.class, "Read right number of bytes", res, 7);

			available = fis.available();
			CheckHelper.check(TestFileInputStream.class, "Available (4)", available, 0);
			available = fis.available();
			CheckHelper.check(TestFileInputStream.class, "Available (5)", available, 0);
		}

		file.delete();
	}
}
