/*
 * Java
 * 
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.constructors;

import java.io.File;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/**
 * Tests for the constructor with one {@link File} and one {@link String} as parameters, {@link File#File(File, String)}.
 *
 * @author pgradot
 *
 */
public class TestFileString {

	/**
	 * Entry point of the program to execute all theses tests.
	 *
	 * @param args
	 *            not used
	 */
	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();
		CheckHelper.startCheck(TestFileString.class);
		test001();
		test002();
		test003();
		test004();

		test101();

		CheckHelper.endCheck(TestFileString.class);
	}

	/**
	 * If child is null, a {@link NullPointerException} should be raised.
	 */
	public static void test001() {
		try {
			File parent = new File("parent");
			@SuppressWarnings("unused")
			File f = new File(parent, null);
			CheckHelper.check(TestFileString.class, "If child is null, a {@link NullPointerException} should be raised", false);
		} catch (NullPointerException e) {
			CheckHelper.check(TestFileString.class, "(Normal to get here)", true);
		}
	}

	/**
	 * If parent is null, it should be equivalent to File(child).
	 */
	public static void test002() {
		String child = "A/B/C/D";
		File one = new File((File) null, child);
		File two = new File(child);

		CheckHelper.check(TestFileString.class, "If parent is null, it should be equivalent to File(child)", one.equals(two));
	}

	/**
	 * If child is absolute, it should be converted to a relative pathname.
	 */
	public static void test003() {
		String childPath = "/A/B/C/D";
		File parent = new File("parent");
		File child = new File(parent, childPath);

		String expected = "parent" + File.separator + "A" + File.separator + "B" + File.separator + "C" + File.separator + "D";
		CheckHelper.check(TestFileString.class, "If child is absolute, it should be converted to a relative pathname", expected.equals(child.getPath()));
	}

	/**
	 * If parent is the empty abstract pathname, the result should be resolved against a system-dependent default directory to be the parent of child.
	 */
	public static void test004() {
		File empty = new File(""); // tested in TestString.test002()
		File child = new File(empty, "test");

		System.out.println(new File("/test").getParentFile());

		CheckHelper.check(TestFileString.class, "If parent is the empty abstract pathname, the result should have a parent", child.getParentFile() != null);
	}

	/**
	 * If both parent and child are 'correct', an abstract path is created by combining the pathname strings.
	 */
	public static void test101() {
		String parent = "A/B/C";
		String child = "D";

		String expected = "A" + File.separator + "B" + File.separator + "C" + File.separator + "D";

		File f = new File(new File(parent), child);

		CheckHelper.check(TestFileString.class, "If both parent and child are 'correct', an abstract path is created by combining the pathname strings",
				expected.equals(f.getPath()));
	}
}
