/*
 * Java
 * 
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.constructors;

import java.io.File;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/**
 * Tests for the constructor with two {@link String}s as parameters, {@link File#File(String, String)}.
 *
 * @author pgradot
 *
 */
public class TestStringString {

	/**
	 * Entry point of the program to execute all theses tests.
	 *
	 * @param args
	 *            not used
	 */
	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();
		CheckHelper.startCheck(TestStringString.class);
		test001();
		test002();
		test003();
		test004();
		test005();

		test101();
		test102();

		CheckHelper.endCheck(TestStringString.class);
	}

	/**
	 * If child is null, a {@link NullPointerException} should be raised.
	 */
	public static void test001() {
		try {
			@SuppressWarnings("unused")
			File f = new File("parent", null);
			CheckHelper.check(TestStringString.class, "If child is null, a {@link NullPointerException} should be raised", false);
		} catch (NullPointerException e) {
			CheckHelper.check(TestStringString.class, "(Normal to get here)", true);
		}
	}

	/**
	 * If parent is null, it should be equivalent to File(child).
	 */
	public static void test002() {
		String child = "A/B/C/D";
		File one = new File((String) null, child);
		File two = new File(child);

		CheckHelper.check(TestStringString.class, "If parent is null, it should be equivalent to File(child)", one.equals(two));
	}

	/**
	 * If child is absolute and parent different from null, it should be converted to a relative pathname.
	 */
	public static void test003() {
		String child = "/A/B/C/D";
		File one = new File("parent", child);
		File two = new File("parent/A/B/C/D");
		CheckHelper.check(TestStringString.class, "If child is aboslute, it should be converted to a relative pathname", one.getAbsolutePath().equals(two.getAbsolutePath()));
	}

	/**
	 * If child is absolute and parent is null, it should not be converted to a relative pathname.
	 */
	public static void test004() {
		String child = "/A/B/C/D";
		File one = new File((String)null, child);
		File two = new File("/A/B/C/D");
		CheckHelper.check(TestStringString.class, "If child is aboslute and parent is null, it should not be converted to a relative pathname", one.getAbsolutePath().equals(two.getAbsolutePath()));
	}

	/**
	 * If parent is the empty string, the result should be resolved against a system-dependent default directory to be the parent of child.
	 */
	public static void test005() {
		String child = "A/B/C/D";
		File one = new File("", child);
		File two = new File("/A/B/C/D");
		CheckHelper.check(TestStringString.class, "If parent is the empty string, the result should be resolved against a system-dependent default directory to be the parent of child.", one.getAbsolutePath().equals(two.getAbsolutePath()));
	}

	/**
	 * If both parent and child are 'correct', an abstract path is created by combining the pathname strings.
	 */
	public static void test101() {
		String parent = "A/B/C";
		String child = "D";

		String expected = "A" + File.separator + "B" + File.separator + "C" + File.separator + "D";

		File f = new File(parent, child);

		CheckHelper
		.check(TestStringString.class,
				"If both parent and child are 'correct', an abstract path is created by combining the pathname strings",
				expected.equals(f.getPath()));
	}

	/**
	 * If parent ends with a name separator, this should not affect the resulting pathname.
	 */
	public static void test102() {
		String parent = "A/B/C/";
		String child = "D";

		String expected = "A" + File.separator + "B" + File.separator + "C" + File.separator + "D";

		File f = new File(parent, child);

		CheckHelper.check(TestStringString.class, "If parent ends with a name separator, this should not affect the resulting pathname",
				expected.equals(f.getPath()));
	}

}
