/*
 * Java
 * 
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.methods;

import java.io.File;
import java.io.IOException;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/**
 * Tests for {@link File#getCanonicalPath()}.
 */
public class TestGetCanonicalPath {

	/**
	 * Entry point of the program to execute all theses tests.
	 *
	 * @param args
	 *            not used
	 */
	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();
		CheckHelper.startCheck(TestGetCanonicalPath.class);
		test001();
		test002();
		test003();
		test004();
		test005();
		test006();
		test007();
		test008();
		test009();

		CheckHelper.endCheck(TestGetCanonicalPath.class);
	}

	/**
	 * The canonical path of the empty abstract pathname should be the property 'user.dir'.
	 */
	public static void test001() {
		try {
			File f = new File("");
			String canonical = f.getCanonicalPath();
			String userDir = new File(System.getProperty("user.dir")).getCanonicalPath();
			CheckHelper.check(TestGetCanonicalPath.class,
					"The canonical path of the empty abstract pathname should be the canonical path of the 'user.dir'",
					userDir.equals(canonical));
		} catch (IOException e) {
			e.printStackTrace();
			CheckHelper.check(TestGetCanonicalPath.class, "Should not raise for the empty abstract pathname", false);
		}
	}

	/**
	 * The canonical path of '.' should be the property 'user.dir'.
	 */
	public static void test002() {
		try {
			File f = new File(".");
			String canonical = f.getCanonicalPath();
			String userDir = new File(System.getProperty("user.dir")).getCanonicalPath();
			CheckHelper.check(TestGetCanonicalPath.class,
					"The canonical path of '.' should be the canonical path of the 'user.dir'",
					userDir.equals(canonical));
		} catch (IOException e) {
			e.printStackTrace();
			CheckHelper.check(TestGetCanonicalPath.class, "Should not raise for '.", false);
		}
	}

	/**
	 * The canonical path of './.' should be the property 'user.dir'.
	 */
	public static void test003() {
		try {
			File f = new File("./.");
			String canonical = f.getCanonicalPath();
			String userDir = new File(System.getProperty("user.dir")).getCanonicalPath();
			CheckHelper.check(TestGetCanonicalPath.class,
					"The canonical path of './.' should be the canonical path of the 'user.dir'",
					userDir.equals(canonical));
		} catch (IOException e) {
			e.printStackTrace();
			CheckHelper.check(TestGetCanonicalPath.class, "Should not raise for './.", false);
		}
	}

	/**
	 * The canonical path of './././' should be the property 'user.dir'.
	 */
	public static void test004() {
		try {
			File f = new File("./././");
			String canonical = f.getCanonicalPath();
			String userDir = new File(System.getProperty("user.dir")).getCanonicalPath();
			CheckHelper.check(TestGetCanonicalPath.class,
					"The canonical path of './././' should be the canonical path of the 'user.dir'",
					userDir.equals(canonical));
		} catch (IOException e) {
			e.printStackTrace();
			CheckHelper.check(TestGetCanonicalPath.class, "Should not raise for './././", false);
		}
	}

	/**
	 * The canonical path of '..' should be the parent of the property 'user.dir'.
	 */
	public static void test005() {

		try {
			File f = new File("..");
			String canonical = f.getCanonicalPath();
			String userDirParent = new File(System.getProperty("user.dir")).getCanonicalFile().getParent();
			if (userDirParent == null) {
				return; // parent does not exist
			}

			CheckHelper.check(TestGetCanonicalPath.class,
					"The canonical path of '..' should be the canonical path of the parent of 'user.dir'",
					userDirParent.equals(canonical)); // FIXME may raise NPE
		} catch (IOException e) {
			e.printStackTrace();
			CheckHelper.check(TestGetCanonicalPath.class, "Should not raise for '..", false);
		}
	}

	/**
	 * The canonical path of '../..//' should be the grand-parent of the property 'user.dir'.
	 */
	public static void test006() {
		try {
			File f = new File("../..//");
			String canonical = f.getCanonicalPath();

			File userDirParent = new File(System.getProperty("user.dir")).getParentFile();
			if(userDirParent == null) {
				return;//no parent :-(
			}

			File userDirGrandParent = userDirParent.getParentFile();
			if (userDirGrandParent == null) {
				return;//grand-parent does not exist :-(
			}
			String userDirGrandParentCanonicalPath = userDirGrandParent.getCanonicalPath();

			CheckHelper.check(TestGetCanonicalPath.class,
					"The canonical path of '../..//' should be the canonical path of the grand-parent of 'user.dir'",
					userDirGrandParentCanonicalPath.equals(canonical));
		} catch (IOException e) {
			e.printStackTrace();
			CheckHelper.check(TestGetCanonicalPath.class, "Should not raise for '../..//", false);
		}
	}

	/**
	 * The canonical path of './.././' should be the parent of the property 'user.dir'.
	 */
	public static void test007() {
		try {
			File f = new File("./.././");
			String canonical = f.getCanonicalPath();
			String userDirParent = new File(System.getProperty("user.dir")).getCanonicalFile().getParent();
			if (userDirParent == null) {
				return; // parent does not exist
			}

			CheckHelper.check(TestGetCanonicalPath.class,
					"The canonical path of './.././.' should be the canonical path of the parent of 'user.dir'",
					userDirParent.equals(canonical));
		} catch (IOException e) {
			e.printStackTrace();
			CheckHelper.check(TestGetCanonicalPath.class, "Should not raise for './.././", false);
		}
	}

	/**
	 * The canonical path of 'foo/../..//' should be the parent of the property 'user.dir'.
	 */
	public static void test008() {
		File foo = new File("foo");
		foo.mkdir();
		try {
			File f = new File("foo/../../");
			String canonical = f.getCanonicalPath();
			String userDirParent = new File(System.getProperty("user.dir")).getCanonicalFile().getParent();
			if (userDirParent == null) {
				return; // parent does not exist
			}
			CheckHelper.check(TestGetCanonicalPath.class,
					"The canonical path of 'foo/../..//' should be the canonical path of the parent of 'user.dir'",
					userDirParent.equals(canonical));
		} catch (IOException e) {
			e.printStackTrace();
			CheckHelper.check(TestGetCanonicalPath.class, "Should not raise for 'foo/../..//", false);
		} finally {
			foo.delete();
		}
	}

	/**
	 * The canonical path of 'foo/bar/..../' should not raise an {@link IOException} or crash.<p>
	 * The behavior is undefined and is different on Linux and Windows.
	 */
	public static void test009(){
		String test = "foo/bar/..../";
		try {
			File f = new File(test);
			String canonical = f.getCanonicalPath();
			CheckHelper.check(TestGetCanonicalPath.class, "The canonical path of '"+test+"' must not crash", canonical.contains("foo"+File.separator+"bar"));
		} catch (IOException e) {
			e.printStackTrace();
			CheckHelper.check(TestGetCanonicalPath.class, "Should not raise IOException for "+test, false);
		}
	}

}
