/*
 * Java
 * 
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.methods;

import java.io.File;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.UtilTests;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/**
 * Tests for {@link File#getParent()}.
 *
 * @author pgradot
 *
 */
public class TestGetParent {

	/**
	 * Entry point of the program to execute all theses tests.
	 *
	 * @param args
	 *            not used
	 */
	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();
		CheckHelper.startCheck(TestGetParent.class);
		test001();
		test002();
		test003();
		test004();
		test005();
		test006();
		test007();
		test008();
		test009();

		test101();

		test201();
		test202();
		test203();
		test204();
		test205();

		CheckHelper.endCheck(TestGetParent.class);
	}

	/**
	 * The parents of the roots should be null.
	 */
	public static void test001() {
		File[] roots = File.listRoots();
		for (int i = 0; i < roots.length; i++) {
			CheckHelper.check(TestGetParent.class, "The parent of root " + i + " should be null", roots[i].getParent() == null);
		}
	}

	/**
	 * The parent of a simple path should be null.
	 */
	public static void test002() {
		File f = new File("A");
		CheckHelper.check(TestGetParent.class, "The parent of a simple path should be null", f.getParent() == null);
	}

	/**
	 * The parent of a simple path that ends with a '/' should be null.
	 */
	public static void test003() {
		File f = new File("A/");
		CheckHelper.check(TestGetParent.class, "The parent of a simple path that ends with a '/' should be null", f.getParent() == null);
	}

	/**
	 * The parent of a simple path that ends with '//' should be null.
	 */
	public static void test004() {
		File f = new File("A//");
		CheckHelper.check(TestGetParent.class, "The parent of a simple path that ends with '//' should be null", f.getParent() == null);
		/*
		 * Having several ending separators is not an issue (compare to having only one) because paths are normalized in File's constructors.
		 */
	}

	/**
	 * The parent of a simple path that ends with a '\\' should be null.
	 */
	public static void test005() {
		File f = new File("A\\");
		CheckHelper.check(TestGetParent.class, "The parent of a simple path that ends with a '\\' should be null", f.getParent() == null);
	}

	/**
	 * The parent of a simple path that ends with '\\\\' should be null.
	 */
	public static void test006() {
		File f = new File("A\\\\");
		CheckHelper.check(TestGetParent.class, "The parent of a simple path that ends with '\\\\' should be null", f.getParent() == null);
		/*
		 * Having several ending separators is not an issue (compare to having only one) because paths are normalized in File's constructors.
		 */
	}

	/**
	 * The parent of a simple path that ends with the name-separator should be null.
	 */
	public static void test007() {
		File f = new File("A" + File.separator);
		CheckHelper.check(TestGetParent.class, "The parent of a simple path that ends with the name-separator should be null", f.getParent() == null);
	}

	/**
	 * The parent of a simple path that ends with several name-separators should be null.
	 */
	public static void test008() {
		File f = new File("A" + File.separator + File.separator + File.separator);
		CheckHelper.check(TestGetParent.class, "The parent of a simple path that ends with several name-separators should be null",
				f.getParent() == null);
		/*
		 * Having several ending separators is not an issue (compare to having only one) because paths are normalized in File's constructors.
		 */
	}

	/**
	 * The parent of the empty abstract pathname should be null.
	 */
	public static void test009() {
		File f = new File(""); // tested in TestString.test002
		CheckHelper.check(TestGetParent.class, "The parent of the empty abstract pathname should be null", f.getParent() == null);
	}

	/**
	 * The parent of a simple path that starts with a name-separator should be the name-separator.
	 *
	 * The case of a path that starts with several name-separator is OS-dependent. It is tested in other classes (such as
	 * {@link TestGetParent_Windows}.
	 */
	public static void test101() {
		File f = new File("/A");
		CheckHelper.check(TestGetParent.class, "The parent of a simple path that starts with a name-separator should be the name-separator (1)",
				File.separator.equals(f.getParent()));

		f = new File("\\B");
		if(UtilTests.onWindows()){
			CheckHelper.check(TestGetParent.class, "The parent of a simple path that starts with a name-separator should be the name-separator (2win)",
					File.separator.equals(f.getParent()));
		}
		else {
			CheckHelper.check(TestGetParent.class, "The parent of a simple path that starts with a name-separator should be null (2unix)",
					f.getParent()==null);
		}

		f = new File(File.separator + "C");
		CheckHelper.check(TestGetParent.class, "The parent of a simple path that starts with a name-separator should be the name-separator (3)",
				File.separator.equals(f.getParent()));
	}

	/**
	 * The parent of "A/B" should be "A".
	 */
	public static void test201() {
		File f = new File("A/B");
		String parentPath = "A";
		CheckHelper.check(TestGetParent.class, "The parent of 'A/B' should be 'A'", parentPath.equals(f.getParent()));
		// FIXME can it be 'A/' or 'A\'?
	}

	/**
	 * An ending name-separator has no effect.
	 */
	public static void test202() {
		String parentPath = "A";

		File f1 = new File("A/B/");
		CheckHelper.check(TestGetParent.class, "An ending name-separator has no effect (1)", parentPath.equals(f1.getParent()));

		File f2 = new File("A/B\\");
		CheckHelper.check(TestGetParent.class, "An ending name-separator has no effect (2)", parentPath.equals(f2.getParent()));

		File f3 = new File("A/B" + File.separator);
		CheckHelper.check(TestGetParent.class, "An ending name-separator has no effect (3)", parentPath.equals(f3.getParent()));
	}

	/**
	 * Several ending name-separator have no effect.
	 */
	public static void test203() {
		String parentPath = "A";

		File f1 = new File("A/B///");
		CheckHelper.check(TestGetParent.class, "Several ending name-separator have no effect (1)", parentPath.equals(f1.getParent()));

		File f2 = new File("A/B\\\\");
		CheckHelper.check(TestGetParent.class, "Several ending name-separator have no effect (2)", parentPath.equals(f2.getParent()));

		File f3 = new File("A/B" + File.separator + File.separator + "\\" + "/");
		if(UtilTests.onWindows()){
			CheckHelper.check(TestGetParent.class, "Several ending name-separator have no effect (3win)", parentPath.equals(f3.getParent()));
		}
		else {
			CheckHelper.check(TestGetParent.class, "Several ending name-separator (3linux)", "A/B".equals(f3.getParent()));
		}
		/*
		 * Having several ending separators is not an issue (compare to having only one) because paths are normalized in File's constructors.
		 */
	}

	/**
	 * The parent of "A/B/C" should be "A + File.separator + B".
	 */
	public static void test204() {
		File f = new File("A/B/C");
		String parentPath = "A" + File.separator + "B";
		CheckHelper.check(TestGetParent.class, "The parent of 'A/B/C' should be 'A + File.separator + B'", parentPath.equals(f.getParent()));
	}

	/**
	 * The parent of 'root[0]/A' should be 'root[0]'.
	 */
	public static void test205() {
		File root0 = File.listRoots()[0];
		File f = new File(root0, "A");
		File parent = f.getParentFile();
		System.out.println("parent = " + parent);
		CheckHelper.check(TestGetParent.class, "The parent of 'root[0]/A' should be 'root[0]'", parent.equals(root0));
	}

}
