/*
 * Java
 *
 * Copyright 2019-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.methods;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/**
 * Tests for {@link FileInputStream#skip(long)} when the platform supports
 * backward skips.
 */
public class TestNegativeSkipWhenSupported {
	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();
		CheckHelper.startCheck(TestNegativeSkipWhenSupported.class);
		File file = createFile();

		if (file != null) {
			try {
				testNegativeSkipWhenSupported(file);
			} catch (IOException e) {
				CheckHelper.check(TestNegativeSkipWhenSupported.class, "IOException", false);
				e.printStackTrace();
			} finally {
				file.delete();
			}
		} else {
			CheckHelper.check(TestNegativeSkipWhenSupported.class, "Cannot create temp file", false);
		}
		CheckHelper.endCheck(TestNegativeSkipWhenSupported.class);
	}

	/**
	 * Creates a temporary file and fill it with data.
	 */
	public static File createFile() {
		try {
			File file = File.createTempFile(TestNegativeSkipWhenSupported.class.getSimpleName(), null);

			try (FileOutputStream fos = new FileOutputStream(file)) {
				fos.write(new byte[256]);
			}

			return file;
		} catch (IOException e) {
			return null;
		}
	}

	/**
	 * Test that skipping a negative number of bytes executes a backward skip
	 * instead of throwing an IOException when this feature is supported.
	 *
	 * IMPORTANT NOTICE: This test can pass only if your platform supports backward
	 * skip on files (using seek for example). It is normally the case. See comments
	 * in {@link FileInputStream#skip(long)}. If your platform doesn't support this
	 * feature, you can safely exclude this test from the tests set.
	 *
	 */
	public static void testNegativeSkipWhenSupported(File f) throws IOException {
		System.out.println("*** TestNegativeSkipWhenSupported.testNegativeSkipWhenSupported ***");
		System.out.println(
				"This test can pass only if your platform supports backward skip on files (using seek for example). It is normally the case. See comments in {@link FileInputStream#skip(long)}. If your platform doesn't support this feature, you can safely exclude this test from the tests set.");

		// Test with a correct negative skip
		try (FileInputStream fis = new FileInputStream(f)) {
			fis.read(new byte[64]);

			int availableBeforeSkip = fis.available();
			try {
				long nbSkipped = fis.skip(-10);
				CheckHelper.check(TestNegativeSkipWhenSupported.class,
						"Skip -10 bytes does not throw an exception but returned value is not correct", nbSkipped, -10);

				int availableAfterSkip = fis.available();
				CheckHelper.check(TestNegativeSkipWhenSupported.class,
						"Skip -10 bytes does not throw an exception but available is not correct", availableAfterSkip,
						availableBeforeSkip + 10);
			} catch (IOException e) {
				CheckHelper.check(TestNegativeSkipWhenSupported.class, "Skip -10 bytes throws an exception", false);
			}
		}

		// Test with a small negative skip (-1) to check potential collisions with
		// common error codes
		try (FileInputStream fis = new FileInputStream(f)) {
			fis.read(new byte[64]);

			int availableBeforeSkip = fis.available();
			try {
				long nbSkipped = fis.skip(-1);
				CheckHelper.check(TestNegativeSkipWhenSupported.class,
						"Skip -1 byte does not throw an exception but returned value is not correct", nbSkipped, -1);

				int availableAfterSkip = fis.available();
				CheckHelper.check(TestNegativeSkipWhenSupported.class,
						"Skip -1 byte does not throw an exception but available is not correct", availableAfterSkip,
						availableBeforeSkip + 1);
			} catch (IOException e) {
				CheckHelper.check(TestNegativeSkipWhenSupported.class, "Skip -1 byte throws an exception", false);
			}
		}

		// Test with a small negative skip (-2) to check potential collisions with
		// common error codes
		try (FileInputStream fis = new FileInputStream(f)) {
			fis.read(new byte[64]);

			int availableBeforeSkip = fis.available();
			try {
				long nbSkipped = fis.skip(-2);
				CheckHelper.check(TestNegativeSkipWhenSupported.class,
						"Skip -2 bytes does not throw an exception but returned value is not correct", nbSkipped, -2);

				int availableAfterSkip = fis.available();
				CheckHelper.check(TestNegativeSkipWhenSupported.class,
						"Skip -2 bytes does not throw an exception but available is not correct", availableAfterSkip,
						availableBeforeSkip + 2);
			} catch (IOException e) {
				CheckHelper.check(TestNegativeSkipWhenSupported.class, "Skip -2 bytes throws an exception", false);
			}
		}
	}
}
