/*
 * Java
 *
 * Copyright 2019-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.methods;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

import ej.bon.Immortals;

/**
 * Tests for {@link FileInputStream#read(byte[], int, int)}
 */
public class TestReadWithOffset {
	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();
		CheckHelper.startCheck(TestReadWithOffset.class);
		File file = createFile();

		if (file != null) {
			try {
				testReadWithOffset(file, false);
				testReadWithOffset(file, true);
				testReadWithoutOffset(file, false);
				testReadWithoutOffset(file, true);
				testReadWithLargeOffset(file, false);
				testReadWithLargeOffset(file, true);
				testReadWithNegativeOffset(file, false);
				testReadWithNegativeOffset(file, true);
				testReadWithNegativeLength(file, false);
				testReadWithNegativeLength(file, true);
			} catch (IOException e) {
				CheckHelper.check(TestReadWithOffset.class, "IOException", false);
				e.printStackTrace();
			} finally {
				file.delete();
			}
		} else {
			CheckHelper.check(TestReadWithOffset.class, "Cannot create temp file", false);
		}
		CheckHelper.endCheck(TestReadWithOffset.class);
	}

	/**
	 * Creates a temporary file and fill it with data.
	 */
	public static File createFile() {
		try {
			File file = File.createTempFile(TestReadWithOffset.class.getSimpleName(), null);

			try (FileOutputStream fos = new FileOutputStream(file)) {
				fos.write("0123456789abcdef".getBytes());
			}

			return file;
		} catch (IOException e) {
			return null;
		}
	}

	/**
	 * Test {@link FileInputStream#read(byte[], int, int)} on a non-immortal array
	 * with a non-zero offset parameter.
	 *
	 * @param useImmortal
	 *            true if the byte array used should be an Immortal, false
	 *            otherwise.
	 */
	public static void testReadWithOffset(File f, boolean useImmortal) throws IOException {
		String str = "xxxxxxxxxx"; // ten x
		String arrayKind = "non-immortal";
		if (useImmortal) {
			arrayKind = "immortal";
		}
		try (FileInputStream fis = new FileInputStream(f)) {
			byte[] dataRead = str.getBytes();
			if (useImmortal) {
				Immortals.setImmortal(dataRead);
			}
			fis.read(dataRead, 1, 9);
			CheckHelper.check(TestReadWithOffset.class,
					"Read from " + arrayKind + " array with offset: string must be \"x012345678\"",
					"x012345678".equals(new String(dataRead)));
		}

		try (FileInputStream fis = new FileInputStream(f)) {
			byte[] dataRead = str.getBytes();
			if (useImmortal) {
				Immortals.setImmortal(dataRead);
			}
			fis.read(dataRead, 5, 5);
			CheckHelper.check(TestReadWithOffset.class,
					"Read from " + arrayKind + " array with offset: string must be \"xxxxx01234\"",
					"xxxxx01234".equals(new String(dataRead)));
		}

		try (FileInputStream fis = new FileInputStream(f)) {
			byte[] dataRead = str.getBytes();
			if (useImmortal) {
				Immortals.setImmortal(dataRead);
			}
			fis.read(dataRead, 1, 1);
			fis.read(dataRead, 3, 1);
			fis.read(dataRead, 5, 1);
			fis.read(dataRead, 7, 1);
			fis.read(dataRead, 9, 1);
			CheckHelper.check(TestReadWithOffset.class,
					"Read from " + arrayKind + " array with offset: string must be \"x0x1x2x3x4\"",
					"x0x1x2x3x4".equals(new String(dataRead)));
		}
	}

	/**
	 * Test {@link FileInputStream#read(byte[], int, int)} with a zero offset
	 * parameter.
	 *
	 * @param useImmortal
	 *            true if the byte array used should be an Immortal, false
	 *            otherwise.
	 */
	public static void testReadWithoutOffset(File f, boolean useImmortal) throws IOException {
		String arrayKind = "non-immortal";
		if (useImmortal) {
			arrayKind = "immortal";
		}
		try (FileInputStream fis = new FileInputStream(f)) {
			byte[] dataRead = "xxxxxxxxxx".getBytes(); // ten x
			if (useImmortal) {
				Immortals.setImmortal(dataRead);
			}
			fis.read(dataRead, 0, 10);
			CheckHelper.check(TestReadWithOffset.class,
					"Read from " + arrayKind + " array without offset: string must be \"0123456789\"",
					"0123456789".equals(new String(dataRead)));
		}
	}

	/**
	 * Test {@link FileInputStream#read(byte[], int, int)} with a too large offset
	 * parameter.
	 *
	 * @param useImmortal
	 *            true if the byte array used should be an Immortal, false
	 *            otherwise.
	 */
	public static void testReadWithLargeOffset(File f, boolean useImmortal) throws IOException {
		String arrayKind = "non-immortal";
		if (useImmortal) {
			arrayKind = "immortal";
		}
		try (FileInputStream fis = new FileInputStream(f)) {
			byte[] dataRead = "xxxxxxxxxx".getBytes(); // ten x
			if (useImmortal) {
				Immortals.setImmortal(dataRead);
			}
			boolean exceptionThrown = false;
			try {
				fis.read(dataRead, 10, 10);
			} catch (IndexOutOfBoundsException e) {
				exceptionThrown = true;
			}
			CheckHelper.check(TestReadWithOffset.class,
					"Read from " + arrayKind + " array with large offset throws an exception ", exceptionThrown);
		}
	}

	/**
	 * Test {@link FileInputStream#read(byte[], int, int)} with a negative offset
	 * parameter.
	 *
	 * @param useImmortal
	 *            true if the byte array used should be an Immortal, false
	 *            otherwise.
	 */
	public static void testReadWithNegativeOffset(File f, boolean useImmortal) throws IOException {
		String arrayKind = "non-immortal";
		if (useImmortal) {
			arrayKind = "immortal";
		}
		try (FileInputStream fis = new FileInputStream(f)) {
			byte[] dataRead = "xxxxxxxxxx".getBytes(); // ten x
			if (useImmortal) {
				Immortals.setImmortal(dataRead);
			}
			boolean exceptionThrown = false;
			try {
				fis.read(dataRead, -10, 10);
			} catch (IndexOutOfBoundsException e) {
				exceptionThrown = true;
			}
			CheckHelper.check(TestReadWithOffset.class,
					"Read from " + arrayKind + " array with negative offset throws an exception ",
					exceptionThrown);
		}
	}

	/**
	 * Test {@link FileInputStream#read(byte[], int, int)} with a negative length
	 * parameter.
	 *
	 * @param useImmortal
	 *            true if the byte array used should be an Immortal, false
	 *            otherwise.
	 */
	public static void testReadWithNegativeLength(File f, boolean useImmortal) throws IOException {
		String arrayKind = "non-immortal";
		if (useImmortal) {
			arrayKind = "immortal";
		}
		try (FileInputStream fis = new FileInputStream(f)) {
			byte[] dataRead = "xxxxxxxxxx".getBytes(); // ten x
			if (useImmortal) {
				Immortals.setImmortal(dataRead);
			}
			boolean exceptionThrown = false;
			try {
				fis.read(dataRead, 10, -10);
			} catch (IndexOutOfBoundsException e) {
				exceptionThrown = true;
			}
			CheckHelper.check(TestReadWithOffset.class,
					"Read from " + arrayKind + " array with negative length throws an exception ",
					exceptionThrown);
		}
	}
}
