/*
 * Java
 * 
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.methods;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/**
 * Test {@link File#setReadOnly()} method.
 * <p/>
 * WARNING: this test fails on Linux if it is executed by the root (on Linux, root can write in read only files).
 */
public class TestSetReadOnly {

	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();

		System.out.println("WARNING: this test fails on Linux if it is executed by the root (on Linux, root can write in read only files).");

		CheckHelper.startCheck(TestSetReadOnly.class);

		try {
			testDelete();
			testCannotWrite();
			testNotWritable();
			testOnDirectory();
		} catch (IOException e) {
			CheckHelper.check(TestSetReadOnly.class, "IOException", false);
			e.printStackTrace();
		}

		CheckHelper.endCheck(TestSetReadOnly.class);
	}

	/**
	 * Test {@link File#setReadOnly()} method on a directory.
	 */
	public static void testOnDirectory() throws IOException{
		File dir = new File(TestSetReadOnly.class.getSimpleName());
		dir.mkdir();

		File newFile1 = null;
		File newFile2;

		boolean readOnly = dir.setReadOnly();
		if(readOnly){
			try{
				newFile1 = new File(dir, "test1");
				newFile1.createNewFile();
				CheckHelper.check(TestSetReadOnly.class, "Create a file in a readonly directory", false);
			}
			catch(IOException e){}
			boolean writable = dir.setWritable(true);
			CheckHelper.check(TestSetReadOnly.class, "Cannot set writable a readonly directory", writable);
		}

		newFile2 = new File(dir, "test2");
		boolean created = newFile2.createNewFile();
		CheckHelper.check(TestSetReadOnly.class, "Cannot create a file in the directory", created);

		if(newFile1 != null){
			newFile1.delete();
		}
		newFile2.delete();
		dir.delete();
	}

	/** Test deletion of a read only file */
	public static void testDelete() throws IOException{
		File f = File.createTempFile(TestSetReadOnly.class.getSimpleName(), null);

		boolean setRO = f.setReadOnly();
		if(setRO){
			boolean deleted = f.delete();

			if(!deleted){
				boolean setWritable = f.setWritable(true);
				CheckHelper.check(TestSetReadOnly.class, "Set writable a read-only file", setWritable);
				deleted = f.delete();
			}

			CheckHelper.check(TestSetReadOnly.class, "Delete file", deleted);
		}

		f.delete();
	}

	/** Test write in a read only file */
	public static void testCannotWrite() throws IOException{
		File f = File.createTempFile(TestSetReadOnly.class.getSimpleName(), null);

		boolean setRO = f.setReadOnly();
		if(setRO){
			try {
				FileOutputStream fos = new FileOutputStream(f);
				CheckHelper.check(TestSetReadOnly.class, "Must not be able to create a FileOutputStream on a read-only file", false);
				fos.close();
			}
			catch(IOException e){

			}

			boolean setWritable = f.setWritable(true);
			CheckHelper.check(TestSetReadOnly.class, "Set writable a read-only file", setWritable);

			final int testValue = 44;
			try {
				FileOutputStream fos = new FileOutputStream(f);
				fos.write(testValue);
				fos.close();
			}
			catch(IOException e){
				CheckHelper.check(TestSetReadOnly.class, "Test write on a writable file", false);
			}

			FileInputStream fis = new FileInputStream(f);
			int data = fis.read();
			fis.close();
			CheckHelper.check(TestSetReadOnly.class, "Read the written value", data == testValue);

			f.delete();
		}
	}

	/**
	 * Test {@link File#canWrite()} after a call to {@link File#setReadOnly()}.
	 */
	public static void testNotWritable() throws IOException{

		File f = File.createTempFile(TestSetReadOnly.class.getSimpleName(), null);
		boolean setRO = f.setReadOnly();
		if(setRO){
			boolean canWrite = f.canWrite();
			CheckHelper.check(TestSetReadOnly.class, "A read only file must not be writable", !canWrite);

			f.setWritable(true);

			canWrite = f.canWrite();
			CheckHelper.check(TestSetReadOnly.class, "A not read only file must be writable", canWrite);
		}

		f.delete();
	}
}
