/*
 * Java
 * 
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.methods;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/**
 * Tests for {@link FileInputStream#skip(long)}
 *
 */
public class TestSkip {

	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();
		CheckHelper.startCheck(TestSkip.class);
		File file = createFile();

		if(file != null){
			try{
				testSkipLimits(file);
				testOverflow(file);
				testNegative(file);
			}
			catch(IOException e){
				CheckHelper.check(TestSkip.class, "IOException", false);
				e.printStackTrace();
			}
			finally{
				file.delete();
			}
		}
		else {
			CheckHelper.check(TestSkip.class, "Cannot create temp file", false);
		}
		CheckHelper.endCheck(TestSkip.class);
	}

	/**
	 * Creates a temporary file and fill it with data.
	 */
	public static File createFile(){
		try {
			File file = File.createTempFile(TestSkip.class.getSimpleName(), null);

			try(FileOutputStream fos = new FileOutputStream(file)){
				fos.write(new byte[256]);
			}

			return file;
		} catch (IOException e) {
			return null;
		}
	}

	/**
	 * Test {@link FileInputStream#skip(long)} with a parameter that can produce an overflow.
	 */
	public static void testOverflow(File f) throws IOException {
		try(FileInputStream fis = new FileInputStream(f)){
			int available = fis.available();
			long skipped = fis.skip(Long.MAX_VALUE);
			CheckHelper.check(TestSkip.class, "Skip with max long", skipped >= available);

			int availableAfterSkip = fis.available();
			CheckHelper.check(TestSkip.class, "Skip with max long: available must be 0", availableAfterSkip, 0);

			int data = fis.read();
			CheckHelper.check(TestSkip.class, "Skip with max long: EOF on read", data, -1);
		}

		try(FileInputStream fis = new FileInputStream(f)){
			int available = fis.available();
			long skipped = fis.skip(Integer.MAX_VALUE);
			CheckHelper.check(TestSkip.class, "Skip with max int", skipped >= available);

			int availableAfterSkip = fis.available();
			CheckHelper.check(TestSkip.class, "Skip with max int: available must be 0", availableAfterSkip, 0);

			int data = fis.read();
			CheckHelper.check(TestSkip.class, "Skip with max int: EOF on read", data, -1);
		}
	}

	/**
	 * Test {@link FileInputStream#skip(long)} until the limit of the file (before the
	 * last byte, last byte, after the last byte).
	 */
	public static void testSkipLimits(File f) throws IOException {
		//Skip all bytes
		try(FileInputStream fis = new FileInputStream(f)){
			int available = fis.available();
			long skipped = fis.skip(available);
			CheckHelper.check(TestSkip.class, "Skip all: available bytes skipped", skipped, available);

			available = fis.available();
			CheckHelper.check(TestSkip.class, "Skip all: no more available bytes", available, 0);

			int data = fis.read();
			CheckHelper.check(TestSkip.class, "Skip all: EOF on read", data, -1);
		}

		//Skip all bytes - 1
		try(FileInputStream fis = new FileInputStream(f)){
			int available = fis.available();
			long skipped = fis.skip(available-1);
			CheckHelper.check(TestSkip.class, "Skip all-1: available-1 bytes skipped", skipped, available-1);

			available = fis.available();
			CheckHelper.check(TestSkip.class, "Skip all-1: 1 byte available", available, 1);

			int data = fis.read();
			CheckHelper.check(TestSkip.class, "Skip all-1: no EOF on first read", data != -1);
			data = fis.read();
			CheckHelper.check(TestSkip.class, "Skip all-1: EOF on second read", data, -1);
		}

		//Skip all bytes + 1
		try(FileInputStream fis = new FileInputStream(f)){
			int available = fis.available();
			long skipped = fis.skip(available+1);
			CheckHelper.check(TestSkip.class, "Skip all+1: at least available bytes skipped", skipped >= available);

			int data = fis.read();
			CheckHelper.check(TestSkip.class, "Skip all+1: EOF on read", data, -1);
		}
	}

	/**
	 * Test that skipping a negative number of bytes throws an {@link IOException}
	 * or executes a backward skip.
	 */
	public static void testNegative(File f) throws IOException {

		//Test with a correct negative skip
		try(FileInputStream fis = new FileInputStream(f)){
			fis.read(new byte[64]);

			int availableBeforeSkip = fis.available();
			try{
				long nbSkipped = fis.skip(-10);
				CheckHelper.check(TestSkip.class,
						"Skip -10 bytes does not throw an exception but returned value is not correct", nbSkipped, -10);

				int availableAfterSkip = fis.available();
				CheckHelper.check(TestSkip.class,
						"Skip -10 bytes does not throw an exception but available is not correct", availableAfterSkip,
						availableBeforeSkip + 10);
			}
			catch(IOException e){
				int availableAfterSkip = fis.available();
				CheckHelper.check(TestSkip.class, "Skip -10 bytes throws an exception but available changes", availableAfterSkip, availableBeforeSkip);
			}
		}

		// Test with a too big negative skip
		try(FileInputStream fis = new FileInputStream(f)){
			fis.read(new byte[64]);

			int availableBeforeSkip = fis.available();
			boolean ok = false;
			try{
				fis.skip(-80);
			}
			catch(IOException e){
				ok = true;
			}
			CheckHelper.check(TestSkip.class, "Skip -80 of bytes does not throw an exception", ok);
			int availableAfterSkip = fis.available();
			CheckHelper.check(TestSkip.class, "Skip -80 of bytes throws an exception but available changes", availableAfterSkip, availableBeforeSkip);
		}

		// Test with a small negative skip (-1) to check potential collisions with
		// common error codes
		try (FileInputStream fis = new FileInputStream(f)) {
			fis.read(new byte[64]);

			int availableBeforeSkip = fis.available();
			try {
				long nbSkipped = fis.skip(-1);
				CheckHelper.check(TestSkip.class,
						"Skip -1 byte does not throw an exception but returned value is not correct", nbSkipped, -1);

				int availableAfterSkip = fis.available();
				CheckHelper.check(TestSkip.class,
						"Skip -1 byte does not throw an exception but available is not correct", availableAfterSkip,
						availableBeforeSkip + 1);
			} catch (IOException e) {
				int availableAfterSkip = fis.available();
				CheckHelper.check(TestSkip.class, "Skip -1 byte throws an exception but available changes",
						availableAfterSkip, availableBeforeSkip);
			}
		}

		// Test with a small negative skip (-2) to check potential collisions with
		// common error codes
		try (FileInputStream fis = new FileInputStream(f)) {
			fis.read(new byte[64]);

			int availableBeforeSkip = fis.available();
			try {
				long nbSkipped = fis.skip(-2);
				CheckHelper.check(TestSkip.class,
						"Skip -2 bytes does not throw an exception but returned value is not correct", nbSkipped, -2);

				int availableAfterSkip = fis.available();
				CheckHelper.check(TestSkip.class,
						"Skip -2 bytes does not throw an exception but available is not correct", availableAfterSkip,
						availableBeforeSkip + 2);
			} catch (IOException e) {
				int availableAfterSkip = fis.available();
				CheckHelper.check(TestSkip.class, "Skip -2 bytes throws an exception but available changes",
						availableAfterSkip, availableBeforeSkip);
			}
		}
	}
}
