/*
 * Java
 *
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.methods;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.UtilTests;
import com.microej.fs.tests.helpers.FileSystemCleaner;

import ej.bon.Immortals;

/**
 * Tests for {@link FileOutputStream#write(byte[], int, int)}
 */
public class TestWriteWithOffset {
	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();
		CheckHelper.startCheck(TestWriteWithOffset.class);
		try {
			testWriteWithOffset(false);
			testWriteWithOffset(true);
			testWriteWithoutOffset(false);
			testWriteWithoutOffset(true);
			testWriteWithLargeOffset(false);
			testWriteWithLargeOffset(true);
			testWriteWithNegativeOffset(false);
			testWriteWithNegativeOffset(true);
			testWriteWithNegativeLength(false);
			testWriteWithNegativeLength(true);
		} catch (IOException e) {
			CheckHelper.check(TestWriteWithOffset.class, "IOException", false);
			e.printStackTrace();
		}
		CheckHelper.endCheck(TestWriteWithOffset.class);
	}

	/**
	 * Test {@link FileOutputStream#write(byte[], int, int)} with a non-zero offset
	 * parameter.
	 *
	 * @param useImmortal
	 *            true if the byte array used should be an Immortal, false
	 *            otherwise.
	 */
	public static void testWriteWithOffset(boolean useImmortal) throws IOException {
		File f = File.createTempFile(TestWriteWithOffset.class.getSimpleName(), null);
		byte[] dataWritten = "0123456789abcdef".getBytes();
		String arrayKind = "non-immortal";
		if (useImmortal) {
			Immortals.setImmortal(dataWritten);
			arrayKind = "immortal";
		}
		try (FileOutputStream fos = new FileOutputStream(f)) {
			fos.write(dataWritten, 1, 9);
			fos.flush();
			CheckHelper.check(TestWriteWithOffset.class,
					"Write from " + arrayKind + " array with offset: file must contain \"123456789\"",
					UtilTests.fileContains(f, "123456789"));

			fos.write(dataWritten, 5, 5);
			fos.flush();
			CheckHelper.check(TestWriteWithOffset.class,
					"Write from " + arrayKind + " array with offset: file must contain \"12345678956789\"",
					UtilTests.fileContains(f, "12345678956789"));

			fos.write(dataWritten, 1, 1);
			fos.write(dataWritten, 3, 1);
			fos.write(dataWritten, 5, 1);
			fos.write(dataWritten, 7, 1);
			fos.write(dataWritten, 9, 1);
			fos.flush();
			CheckHelper.check(TestWriteWithOffset.class,
					"Write from " + arrayKind + " array with offset: file must contain \"1234567895678913579\"",
					UtilTests.fileContains(f, "1234567895678913579"));
		}
		f.delete();
	}

	/**
	 * Test {@link FileOutputStream#write(byte[], int, int)} with a zero offset
	 * parameter.
	 *
	 * @param useImmortal
	 *            true if the byte array used should be an Immortal, false
	 *            otherwise.
	 */
	public static void testWriteWithoutOffset(boolean useImmortal) throws IOException {
		File f = File.createTempFile(TestWriteWithOffset.class.getSimpleName(), null);
		byte[] dataWritten = "0123456789abcdef".getBytes();
		String arrayKind = "non-immortal";
		if (useImmortal) {
			Immortals.setImmortal(dataWritten);
			arrayKind = "immortal";
		}
		try (FileOutputStream fos = new FileOutputStream(f)) {
			fos.write(dataWritten, 0, 16);
		}
		CheckHelper.check(TestWriteWithOffset.class,
				"Write from " + arrayKind + " array without offset: file must contain \"0123456789abcdef\"",
				UtilTests.fileContains(f, "0123456789abcdef"));
		f.delete();
	}

	/**
	 * Test {@link FileOutputStream#write(byte[], int, int)} with a too large offset
	 * parameter.
	 *
	 * @param useImmortal
	 *            true if the byte array used should be an Immortal, false
	 *            otherwise.
	 */
	public static void testWriteWithLargeOffset(boolean useImmortal) throws IOException {
		File f = File.createTempFile(TestWriteWithOffset.class.getSimpleName(), null);
		byte[] dataWritten = "0123456789abcdef".getBytes();
		String arrayKind = "non-immortal";
		if (useImmortal) {
			Immortals.setImmortal(dataWritten);
			arrayKind = "immortal";
		}
		try (FileOutputStream fos = new FileOutputStream(f)) {
			boolean exceptionThrown = false;
			try {
				fos.write(dataWritten, 10, 10);
			} catch (IndexOutOfBoundsException e) {
				exceptionThrown = true;
			}
			CheckHelper.check(TestWriteWithOffset.class,
					"Write from " + arrayKind + " array with large offset throws an exception ",
					exceptionThrown);
		}
		f.delete();
	}

	/**
	 * Test {@link FileOutputStream#write(byte[], int, int)} with a negative offset
	 * parameter.
	 *
	 * @param useImmortal
	 *            true if the byte array used should be an Immortal, false
	 *            otherwise.
	 */
	public static void testWriteWithNegativeOffset(boolean useImmortal) throws IOException {
		File f = File.createTempFile(TestWriteWithOffset.class.getSimpleName(), null);
		byte[] dataWritten = "0123456789abcdef".getBytes();
		String arrayKind = "non-immortal";
		if (useImmortal) {
			Immortals.setImmortal(dataWritten);
			arrayKind = "immortal";
		}
		try (FileOutputStream fos = new FileOutputStream(f)) {
			boolean exceptionThrown = false;
			try {
				fos.write(dataWritten, -10, 10);
			} catch (IndexOutOfBoundsException e) {
				exceptionThrown = true;
			}
			CheckHelper.check(TestWriteWithOffset.class,
					"Write from " + arrayKind + " array with negative offset throws an exception ",
					exceptionThrown);
		}
		f.delete();
	}

	/**
	 * Test {@link FileOutputStream#write(byte[], int, int)} with a negative length
	 * parameter.
	 *
	 * @param useImmortal
	 *            true if the byte array used should be an Immortal, false
	 *            otherwise.
	 */
	public static void testWriteWithNegativeLength(boolean useImmortal) throws IOException {
		File f = File.createTempFile(TestWriteWithOffset.class.getSimpleName(), null);
		byte[] dataWritten = "0123456789abcdef".getBytes();
		String arrayKind = "non-immortal";
		if (useImmortal) {
			Immortals.setImmortal(dataWritten);
			arrayKind = "immortal";
		}
		try (FileOutputStream fos = new FileOutputStream(f)) {
			boolean exceptionThrown = false;
			try {
				fos.write(dataWritten, 10, -10);
			} catch (IndexOutOfBoundsException e) {
				exceptionThrown = true;
			}
			CheckHelper.check(TestWriteWithOffset.class,
					"Write from " + arrayKind + " array with negative length throws an exception ",
					exceptionThrown);
		}
		f.delete();
	}
}
