/*
 * Java
 * 
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.scenarios;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.UtilTests;
import com.microej.fs.tests.constructors.TestFileInputStream;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/** Test {@link FileOutputStream}. */
public class TestFileOutputStream {

	/** Directory that exists. */
	private static File ExistingDir;
	/** Directory that does not exist. */
	private static File MissingDir;
	/** File that exists. */
	private static File ExistingFile;

	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();

		CheckHelper.startCheck(TestFileOutputStream.class);
		try{
			initTests();
			testConstructorString();
			testConstructorFile();
			testConstructorStringBooleanAppend();
			testConstructorStringBooleanNotAppend();
			testConstructorFileBooleanAppend();
			testConstructorFileBooleanNotAppend();
			testWrite();
			testWriteInvalid();
			cleanTests();
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "IOException", false);
			e.printStackTrace();
		}

		CheckHelper.endCheck(TestFileOutputStream.class);
	}

	private static void initTests() throws IOException{
		ExistingDir = new File("existing"+TestFileOutputStream.class.getSimpleName());
		ExistingDir.mkdirs();
		CheckHelper.check(TestFileOutputStream.class, "Existing dir must exist", ExistingDir.exists());
		//Delete all the file of the directory
		for(File files : ExistingDir.listFiles()){
			files.delete();
		}

		MissingDir = new File("missing"+TestFileOutputStream.class.getSimpleName());
		CheckHelper.check(TestFileOutputStream.class, "Missing dir must not exist", !MissingDir.exists());

		ExistingFile = File.createTempFile(TestFileOutputStream.class.getSimpleName(), null);
		CheckHelper.check(TestFileOutputStream.class, "Existing file must exist", ExistingFile.exists());
	}

	private static void cleanTests() throws IOException{
		ExistingFile.delete();
		//Delete all the file of the directory
		for(File files : ExistingDir.listFiles()){
			files.delete();
		}
		ExistingDir.delete();
	}

	/** Test {@link FileOutputStream#FileOutputStream(String)} */
	public static void testConstructorString() throws IOException{

		try {
			String filePath = ExistingDir.getAbsolutePath()+File.separator+"testString1";
			FileOutputStream fos = new FileOutputStream(filePath);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testString Created file must be empty", UtilTests.fileIsEmpty(new File(filePath)));

			filePath = ExistingDir.getAbsolutePath()+File.separator+"testString2";
			fos = new FileOutputStream(filePath);
			String content = "yopyep";
			fos.write(content.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testString File content is ok (1)", UtilTests.fileContains(new File(filePath), content));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testString IOException(1)", false);
		}

		try {
			// Fill existing file with data
			FileOutputStream fos = new FileOutputStream(ExistingFile.getAbsolutePath());
			fos.write(44);
			fos.close();
			// Open existing file without append should empty the file
			fos = new FileOutputStream(ExistingFile.getAbsolutePath());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testString Open existing file without append must empty the file", UtilTests.fileIsEmpty(ExistingFile));

			fos = new FileOutputStream(ExistingFile.getAbsolutePath());
			String content = "yopyep";
			fos.write(content.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testString File content is ok (2)", UtilTests.fileContains(ExistingFile, content));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testString IOException(2)", false);
		}

		String filePath = MissingDir.getAbsolutePath()+File.separator+"testString3";
		try {
			FileOutputStream fos = new FileOutputStream(filePath);
			CheckHelper.check(TestFileOutputStream.class, "testString No IOException(3)", false);
		}
		catch(IOException e){
		}
		CheckHelper.check(TestFileOutputStream.class, "testString File should not have been created", !new File(filePath).exists());
	}


	/** Test {@link FileOutputStream#FileOutputStream(File)} */
	public static void testConstructorFile() throws IOException{

		try {
			File file = new File(ExistingDir.getAbsolutePath(), "testFile1");
			FileOutputStream fos = new FileOutputStream(file);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testFile Created file must be empty", UtilTests.fileIsEmpty(file));

			file = new File(ExistingDir.getAbsolutePath(), "testFile2");
			fos = new FileOutputStream(file);
			String content = "yopyep";
			fos.write(content.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testFile File content is ok (1)", UtilTests.fileContains(file, content));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testFile IOException(1)", false);
		}

		try {
			// Fill existing file with data
			FileOutputStream fos = new FileOutputStream(ExistingFile);
			fos.write(44);
			fos.close();
			// Open existing file without append should empty the file
			fos = new FileOutputStream(ExistingFile);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testFile Open existing file without append must empty the file", UtilTests.fileIsEmpty(ExistingFile));

			fos = new FileOutputStream(ExistingFile);
			String content = "yopyep";
			fos.write(content.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testFile File content is ok (2)", UtilTests.fileContains(ExistingFile, content));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testFile IOException(2)", false);
		}

		File file = new File(MissingDir, "testFile3");
		try {
			FileOutputStream fos = new FileOutputStream(file);
			CheckHelper.check(TestFileOutputStream.class, "testFile No IOException(3)", false);
		}
		catch(IOException e){
		}
		CheckHelper.check(TestFileOutputStream.class, "testFile File should not have been created", !file.exists());
	}

	/** Test {@link FileOutputStream#FileOutputStream(String, boolean)} with append at true */
	public static void testConstructorStringBooleanAppend() throws IOException{

		try {
			String filePath = ExistingDir.getAbsolutePath()+File.separator+"testConstructorStringBooleanAppend1";
			FileOutputStream fos = new FileOutputStream(filePath, true);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBoolean1 Created file must be empty", UtilTests.fileIsEmpty(new File(filePath)));

			filePath = ExistingDir.getAbsolutePath()+File.separator+"testConstructorStringBooleanAppend2";
			fos = new FileOutputStream(filePath, true);
			String content = "yopyep";
			fos.write(content.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBoolean1 File content is ok (1)", UtilTests.fileContains(new File(filePath), content));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBoolean1 IOException(1)", false);
		}

		try {
			// Fill existing file with data
			String content = "top";
			FileOutputStream fos = new FileOutputStream(ExistingFile.getAbsolutePath());
			fos.write(content.getBytes());
			fos.close();
			// Open existing file with append should keep file content
			fos = new FileOutputStream(ExistingFile.getAbsolutePath(), true);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBoolean1 Open existing file with append must keep file content",  UtilTests.fileContains(ExistingFile, content));

			fos = new FileOutputStream(ExistingFile.getAbsolutePath(), true);
			String appendedContent = "yopyep";
			fos.write(appendedContent.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBoolean1 File content is ok (2)", UtilTests.fileContains(ExistingFile, content+appendedContent));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBoolean1 IOException(2)", false);
		}

		String filePath = MissingDir.getAbsolutePath()+File.separator+"testString3";
		try {
			FileOutputStream fos = new FileOutputStream(filePath, true);
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBoolean1 No IOException(3)", false);
		}
		catch(IOException e){
		}
		CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBoolean1 File should not have been created", !new File(filePath).exists());
	}

	/** Test {@link FileOutputStream#FileOutputStream(String, boolean)} with append at false */
	public static void testConstructorStringBooleanNotAppend() throws IOException{
		try {
			String filePath = ExistingDir.getAbsolutePath()+File.separator+"testConstructorStringBooleanNotAppend1";
			FileOutputStream fos = new FileOutputStream(filePath, false);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBooleanNotAppend Created file must be empty", UtilTests.fileIsEmpty(new File(filePath)));

			filePath = ExistingDir.getAbsolutePath()+File.separator+"testConstructorStringBooleanNotAppend2";
			fos = new FileOutputStream(filePath, false);
			String content = "yopyep";
			fos.write(content.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBooleanNotAppend File content is ok (1)", UtilTests.fileContains(new File(filePath), content));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBooleanNotAppend IOException(1)", false);
		}

		try {
			// Fill existing file with data
			FileOutputStream fos = new FileOutputStream(ExistingFile.getAbsolutePath());
			fos.write(44);
			fos.close();
			// Open existing file without append should empty the file
			fos = new FileOutputStream(ExistingFile.getAbsolutePath(), false);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBooleanNotAppend Open existing file without append must empty the file", UtilTests.fileIsEmpty(ExistingFile));

			fos = new FileOutputStream(ExistingFile.getAbsolutePath(), false);
			String content = "yopyep";
			fos.write(content.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBooleanNotAppend File content is ok (2)", UtilTests.fileContains(ExistingFile, content));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBooleanNotAppend IOException(2)", false);
		}

		String filePath = MissingDir.getAbsolutePath()+File.separator+"testString3";
		try {
			FileOutputStream fos = new FileOutputStream(filePath, false);
			CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBooleanNotAppend No IOException(3)", false);
		}
		catch(IOException e){
		}
		CheckHelper.check(TestFileOutputStream.class, "testConstructorStringBooleanNotAppend File should not have been created", !new File(filePath).exists());
	}

	/** Test {@link FileOutputStream#FileOutputStream(File, boolean)} with append at true */
	public static void testConstructorFileBooleanAppend() throws IOException{

		try {
			File file = new File(ExistingDir.getAbsolutePath(), "testConstructorFileBooleanAppend1");
			FileOutputStream fos = new FileOutputStream(file, true);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanAppend Created file must be empty", UtilTests.fileIsEmpty(file));

			file = new File(ExistingDir.getAbsolutePath(), "testConstructorFileBooleanAppend2");
			fos = new FileOutputStream(file, true);
			String content = "yopyep";
			fos.write(content.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanAppend File content is ok (1)", UtilTests.fileContains(file, content));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanAppend IOException(1)", false);
		}

		try {
			// Fill existing file with data
			String content = "top";
			FileOutputStream fos = new FileOutputStream(ExistingFile);
			fos.write(content.getBytes());
			fos.close();
			// Open existing file with append keep file content
			fos = new FileOutputStream(ExistingFile, true);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanAppend Open existing file with append must keep file content", UtilTests.fileContains(ExistingFile, content));

			fos = new FileOutputStream(ExistingFile, true);
			String appendedContent = "yopyep";
			fos.write(appendedContent.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanAppend File content is ok (2)", UtilTests.fileContains(ExistingFile, content+appendedContent));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanAppend IOException(2)", false);
		}

		File file = new File(MissingDir, "testFile3");
		try {
			FileOutputStream fos = new FileOutputStream(file, true);
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanAppend No IOException(3)", false);
		}
		catch(IOException e){
		}
		CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanAppend File should not have been created", !file.exists());
	}

	/** Test {@link FileOutputStream#FileOutputStream(File, boolean)} with append at false */
	public static void testConstructorFileBooleanNotAppend() throws IOException{

		try {
			File file = new File(ExistingDir.getAbsolutePath(), "testConstructorFileNotAppend1");
			FileOutputStream fos = new FileOutputStream(file, false);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanNotAppend Created file must be empty", UtilTests.fileIsEmpty(file));

			file = new File(ExistingDir.getAbsolutePath(), "testConstructorFileNotAppend2");
			fos = new FileOutputStream(file, false);
			String content = "yopyep";
			fos.write(content.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanNotAppend File content is ok (1)", UtilTests.fileContains(file, content));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanNotAppend IOException(1)", false);
		}

		try {
			// Fill existing file with data
			FileOutputStream fos = new FileOutputStream(ExistingFile);
			fos.write(44);
			fos.close();
			// Open existing file without append should empty the file
			fos = new FileOutputStream(ExistingFile, false);
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanNotAppend Open existing file without append must empty the file", UtilTests.fileIsEmpty(ExistingFile));

			fos = new FileOutputStream(ExistingFile, false);
			String content = "yopyep";
			fos.write(content.getBytes());
			fos.close();
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanNotAppend File content is ok (2)", UtilTests.fileContains(ExistingFile, content));
		}
		catch(IOException e){
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanNotAppend IOException(2)", false);
		}

		File file = new File(MissingDir, "testFile3");
		try {
			FileOutputStream fos = new FileOutputStream(file, false);
			CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanNotAppend No IOException(3)", false);
		}
		catch(IOException e){
		}
		CheckHelper.check(TestFileOutputStream.class, "testConstructorFileBooleanNotAppend File should not have been created", !file.exists());
	}

	/** Test writing in a {@link FileOutputStream}. */
	public static void testWrite() throws IOException{
		File f = File.createTempFile(TestFileOutputStream.class.getSimpleName(), null);
		FileOutputStream fos;

		fos = new FileOutputStream(f);
		fos.write('a');
		byte[] data = "toubidou".getBytes();
		fos.write(data);
		fos.write(new byte[0]);
		fos.write(data, 3, 5);
		fos.write('b');
		fos.close();
		CheckHelper.check(TestFileOutputStream.class, "testWrite Check file content fails(1)", UtilTests.fileContains(f, "atoubidoubidoub"));

		fos = new FileOutputStream(f, true);
		fos.write('x');
		fos.write("yahoo".getBytes());
		fos.close();
		CheckHelper.check(TestFileOutputStream.class, "testWrite Check file content fails(2)", UtilTests.fileContains(f, "atoubidoubidoubxyahoo"));

	}

	/** Tests {@link FileOutputStream#write(byte[],int,int)} with invalid length, offset, etc. */
	public static void testWriteInvalid() throws IOException {
		File file = File.createTempFile(TestFileInputStream.class.getSimpleName(), null);

		try(FileOutputStream fos = new FileOutputStream(file)){

			byte[] data = new byte[5];
			try{
				fos.write(data, 6, 1);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) invalid offset (1) ", false);
			}
			catch(IndexOutOfBoundsException e){
				// ok
			}

			try{
				fos.write(data, 1, -1);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) negative length (1) ", false);
			}
			catch(IndexOutOfBoundsException e){
				// ok
			}

			try{
				fos.write(data, 1, 6);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) invalid length (1) ", false);
			}
			catch(IndexOutOfBoundsException e){
				// ok
			}

			try{
				fos.write(data, 1, Integer.MAX_VALUE);
				CheckHelper.check(TestFileInputStream.class, "Read(byte[], int, int) overflow (1) ", false);
			}
			catch(IndexOutOfBoundsException e){
				// ok
			}

		}
		file.delete();
	}
}
