/*
 * Java
 * 
 * Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.fs.tests.scenarios;

import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.is2t.testsuite.support.CheckHelper;
import com.microej.fs.tests.helpers.FileSystemCleaner;

/**
 * Tests that file descriptors are correctly released.
 */
public class TestMemory {

	public static void main(String[] args) {
		FileSystemCleaner.cleanFS();

		CheckHelper.startCheck(TestMemory.class);
		try {
			testFileOutputStreamsOnError();
			testFileInputStreamsOnError();
			testTooManyOpenFiles();
			testValidDirectory();
			testInvalidDirectory();
		} catch (IOException e) {
			CheckHelper.check(TestMemory.class, "IOException", false);
			e.printStackTrace();
		}

		CheckHelper.endCheck(TestMemory.class);
	}

	/**
	 * Tests that there is no memory leak when {@link FileOutputStream} creation fails.
	 */
	public static void testFileOutputStreamsOnError() throws IOException {
		File invalidFile = new File("this/file/does/not/exists");
		for(int i=1000 ; --i>=0 ;){
			try{
				FileOutputStream fos = new FileOutputStream(invalidFile);
				CheckHelper.check(TestMemory.class, "IOException must occur", false);
			}
			catch(IOException e){}
		}

		CheckHelper.check(TestMemory.class, "Opening file must succeed (output error)", iCanOpenAFile());
	}

	/**
	 * Tests that there is no memory leak when {@link FileInputStream} creation fails.
	 */
	public static void testFileInputStreamsOnError() throws IOException {
		File invalidFile = new File("this/file/does/not/exists");
		for(int i=1000 ; --i>=0 ;){
			try{
				FileInputStream fis = new FileInputStream(invalidFile);
				CheckHelper.check(TestMemory.class, "IOException must occur", false);
			}
			catch(IOException e){}
		}

		CheckHelper.check(TestMemory.class, "Opening file must succeed (input error)", iCanOpenAFile());
	}

	/**
	 * Tests that creating a memory leak is recoverable.
	 */
	public static void testTooManyOpenFiles() throws IOException {
		List<FileDesc> fileDescs = new ArrayList<>();
		final int maxCreatedFiles = 300;

		int uid = 0;
		try{
			for(int i=maxCreatedFiles ; --i>=0 ;){
				File file = File.createTempFile(TestMemory.class.getSimpleName(), "."+(uid++));
				file.createNewFile();
				FileInputStream fis = new FileInputStream(file);
				fileDescs.add(new FileDesc(file, fis));
				if(i%50 == 0){
					CheckHelper.showActivity();
				}
			}
		}
		catch(IOException e){
			//Too many open files
		}

		int nbOpenedFiles = fileDescs.size();
		System.out.println(nbOpenedFiles+" files opened.");

		// Close and delete all the files
		for(FileDesc fileDesc : fileDescs){
			fileDesc.clean();
		}

		// Now we must be able to create nbOpenedFiles FileInputStream
		fileDescs.clear();
		try{
			for(int i=maxCreatedFiles ; --i>=0 ;){
				File file = File.createTempFile(TestMemory.class.getSimpleName(), "."+(uid++));
				file.createNewFile();
				FileInputStream fis = new FileInputStream(file);
				fileDescs.add(new FileDesc(file, fis));
				if(i%50 == 0){
					CheckHelper.showActivity();
				}
			}
		}
		catch(IOException e){
			//Too many open files
		}

		CheckHelper.check(TestMemory.class, "Too many open files must be recoverable", fileDescs.size() == nbOpenedFiles);

		// Close and delete all the files
		for(FileDesc fileDesc : fileDescs){
			fileDesc.clean();
		}
	}

	static class FileDesc {
		public File file;
		public Closeable stream;

		public FileDesc(File file, Closeable stream){
			this.file = file;
			this.stream = stream;
		}

		public void clean() throws IOException{
			stream.close();
			file.delete();
		}
	}

	/**
	 * Tests that listing files in a directory does not create a memory leak.
	 */
	public static void testInvalidDirectory() throws IOException {
		final int nbListTest = 1000; // Number of call of File.list()

		File directory = new File(TestMemory.class.getSimpleName()+"_testInValidDirectory");
		CheckHelper.check(TestMemory.class, "Directory must not exist", !directory.exists());

		for(int i=nbListTest ; --i>=0 ;){
			String[] sfiles = directory.list();
			CheckHelper.check(TestMemory.class, "List must be null or empty", sfiles == null || sfiles.length==0);


			File[] files = directory.listFiles();
			CheckHelper.check(TestMemory.class, "ListFiles must be null", files == null || files.length==0);
		}

		CheckHelper.check(TestMemory.class, "Opening file must succeed (invalid dir)", iCanOpenAFile());
	}

	/**
	 * Tests that listing files in an invalid directory does not create a memory leak.
	 */
	public static void testValidDirectory() throws IOException {
		final int nbFiles = 64; // Number of files created in the tested directory
		final int nbListTest = 100; // Number of call of File.list()

		File directory = new File(TestMemory.class.getSimpleName()+"_testValidDirectory");
		directory.mkdirs();

		// Clean the directory
		File[] currentFiles = directory.listFiles();
		for(File file : currentFiles){
			file.delete();
		}

		for(int i=nbFiles ; --i>=0 ;){
			File.createTempFile(TestMemory.class.getSimpleName(), null, directory);
		}

		CheckHelper.showActivity();

		for(int i=nbListTest ; --i>=0 ;){
			String[] sfiles = directory.list();
			CheckHelper.check(TestMemory.class, "List must not be null", sfiles != null);
			if(sfiles != null){
				CheckHelper.check(TestMemory.class, "List must contain "+nbFiles+" files; returns "+sfiles.length, sfiles.length == nbFiles);
			}

			File[] files = directory.listFiles();
			CheckHelper.check(TestMemory.class, "ListFiles must not be null", files != null);
			if(files != null){
				CheckHelper.check(TestMemory.class, "ListFiles must contain "+nbFiles+" files; returns "+sfiles.length, files.length == nbFiles);
			}

			if(i%20 == 0){
				CheckHelper.showActivity();
			}
		}

		CheckHelper.showActivity();

		CheckHelper.check(TestMemory.class, "Opening file must succeed (valid dir)", iCanOpenAFile());

		File[] files = directory.listFiles();
		for(File file : files){
			file.delete();
		}
		directory.delete();
	}

	/**
	 * Test if we can create a file
	 * @return true if a file can be opened, false otherwise.
	 */
	private static boolean iCanOpenAFile(){
		try{
			File existingFile = File.createTempFile(TestMemory.class.getSimpleName(), null);
			FileInputStream fis = new FileInputStream(existingFile);
			fis.read();
			fis.close();
			existingFile.delete();
		}
		catch(IOException e){
			return false;
		}
		return true;
	}
}
